@@ -51,18 +51,22 @@ class Inference(MaitaiClient):
         return QueueIterable(q, timeout=timeout)
 
     @classmethod
-    def create_chat_storage_request(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
-                                    final_chunk: Optional[ChatCompletionChunk], content: str, timing: RequestTimingMetric):
-        inference_request = cls.create_inference_request(application_ref_name, session_id, reference_id, intent, False, False, completion_params, None, False, '')
+    def store_chat_response(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
+                            final_chunk: Optional[ChatCompletionChunk], content: str, timing: RequestTimingMetric, metadata: dict):
+        inference_request = ChatCompletionRequest(
+            application_ref_name=application_ref_name,
+            session_id=session_id,
+            reference_id=reference_id,
+            action_type=intent,
+            apply_corrections=False,
+            evaluation_enabled=False,
+            params=completion_params,
+            auth_keys=config.auth_keys,
+            metadata=metadata,
+        )
         if final_chunk:
             chat_completion_response = chat_completion_chunk_to_response(final_chunk, content)
-
-        return ChatStorageRequest(chat_completion_request=inference_request, chat_completion_response=chat_completion_response, timing_metrics=timing)
-
-    @classmethod
-    def store_chat_response(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
-                            final_chunk: Optional[ChatCompletionChunk], content: str, timing: RequestTimingMetric):
-        chat_storage_request = cls.create_chat_storage_request(session_id, reference_id, intent, application_ref_name, completion_params, chat_completion_response, final_chunk, content, timing)
+        chat_storage_request = ChatStorageRequest(chat_completion_request=inference_request, chat_completion_response=chat_completion_response, timing_metrics=timing)
         cls.run_async(cls.send_storage_request_async(chat_storage_request))
 
     @classmethod
@@ -75,9 +79,15 @@ class Inference(MaitaiClient):
                 'x-api-key': config.api_key,
                 'x-client-version': version,
             }
-            response = requests.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE), stream=True, timeout=timeout or 120)
-            if response.status_code != 200:
-                raise MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
+            try:
+                response = requests.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE), stream=True, timeout=timeout or 120)
+                if response.status_code != 200:
+                    cls.log_error(response.text, url)
+                    raise MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
+            except Exception as e:
+                cls.log_error(str(e), url)
+                raise MaitaiConnectionError(f"Failed to send inference request. Error: {e}")
+
             try:
                 for line in response.iter_lines():
                     if line:
@@ -100,34 +110,43 @@ class Inference(MaitaiClient):
             'x-api-key': config.api_key,
             'x-client-version': version,
         }
-        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False), timeout=ClientTimeout(timeout or 120)) as session:
-            async with session.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE)) as response:
-                if response.status != 200:
-                    error_text = await response.text()
-                    exception = MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status}. Error: {error_text}")
-                    if chunk_queue:
-                        chunk_queue.put(exception)
-                    if async_chunk_queue:
-                        await async_chunk_queue.put(exception)
-                    return
-                async for line in response.content:
-                    if line:
-                        inference_response: InferenceStreamResponse = InferenceStreamResponse().from_json(line)
-                        if inference_response.keep_alive:
-                            continue
+        try:
+            async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False), timeout=ClientTimeout(timeout or 120)) as session:
+                async with session.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE)) as response:
+                    if response.status != 200:
+                        error_text = await response.text()
+                        cls.log_error(error_text, url)
+                        exception = MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status}. Error: {error_text}")
                         if chunk_queue:
-                            chunk_queue.put(inference_response)
+                            chunk_queue.put(exception)
                         if async_chunk_queue:
-                            await async_chunk_queue.put(inference_response)
-                        if inference_response.evaluate_response and evaluation_callback:
-                            try:
-                                evaluation_callback(inference_response.evaluate_response)
-                            except:
-                                traceback.print_exc()
-                if chunk_queue:
-                    chunk_queue.put(StopIteration())
-                if async_chunk_queue:
-                    await async_chunk_queue.put(StopIteration())
+                            await async_chunk_queue.put(exception)
+                        return
+                    async for line in response.content:
+                        if line:
+                            inference_response: InferenceStreamResponse = InferenceStreamResponse().from_json(line)
+                            if inference_response.keep_alive:
+                                continue
+                            if chunk_queue:
+                                chunk_queue.put(inference_response)
+                            if async_chunk_queue:
+                                await async_chunk_queue.put(inference_response)
+                            if inference_response.evaluate_response and evaluation_callback:
+                                try:
+                                    evaluation_callback(inference_response.evaluate_response)
+                                except:
+                                    traceback.print_exc()
+                    if chunk_queue:
+                        chunk_queue.put(StopIteration())
+                    if async_chunk_queue:
+                        await async_chunk_queue.put(StopIteration())
+        except Exception as e:
+            exception = MaitaiConnectionError(f"Failed to send inference request: {str(e)}")
+            await cls.increment_error(str(e), url)
+            if chunk_queue:
+                chunk_queue.put(exception)
+            if async_chunk_queue:
+                await async_chunk_queue.put(exception)
 
     @classmethod
     async def send_storage_request_async(cls, chat_storage_request: ChatStorageRequest):
@@ -141,7 +160,8 @@ class Inference(MaitaiClient):
         try:
             async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
                 return await session.put(url, headers=headers, data=chat_storage_request.to_json(casing=Casing.SNAKE))
-        except:
+        except Exception as e:
+            await cls.increment_error(str(e), url)
             pass
 
     @classmethod
