@@ -13,13 +13,35 @@ from openai.types.chat import ChatCompletionStreamOptionsParam
 import maitai
 from maitai._config import config
 from maitai._evaluator import Evaluator
-from maitai._inference import InferenceException, InferenceWarning, MaitaiConnectionError
-from maitai._types import Body, Headers, MaitaiChunk, MaitaiCompletion, Query, ToolFunction
-from maitai._utils import convert_open_ai_chat_completion_chunk, convert_openai_chat_completion, get_chat_completion_params, required_args
+from maitai._inference import (
+    InferenceException,
+    InferenceWarning,
+    MaitaiConnectionError,
+)
+from maitai._types import (
+    Body,
+    Headers,
+    MaitaiChunk,
+    MaitaiCompletion,
+    Query,
+    ToolFunction,
+)
+from maitai._utils import (
+    convert_open_ai_chat_completion_chunk,
+    convert_openai_chat_completion,
+    get_chat_completion_params,
+    required_args,
+)
 from maitai.tools import Tools
 from maitai_common.utils.proto_utils import openai_messages_to_proto
 from maitai_common.utils.types import EvaluateCallback
-from maitai_gen.chat import ChatCompletionParams, ChatCompletionRequest, ChatMessage, ClientParams, EvaluationContentType
+from maitai_gen.chat import (
+    ChatCompletionParams,
+    ChatCompletionRequest,
+    ChatMessage,
+    ClientParams,
+    EvaluationContentType,
+)
 from maitai_gen.config import InferenceLocations
 from maitai_gen.inference import InferenceStreamResponse
 from maitai_gen.metric import RequestTimingMetric
@@ -145,6 +167,7 @@ class Completions:
         logit_bias: Union[Optional[Dict[str, int]], openai.NotGiven] = openai.NOT_GIVEN,
         logprobs: Union[Optional[bool], openai.NotGiven] = openai.NOT_GIVEN,
         max_tokens: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
+        max_completion_tokens: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
         n: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
         presence_penalty: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
         response_format: Union[
@@ -233,6 +256,8 @@ class Completions:
             logprobs = maitai_config.logprobs
         if max_tokens == openai.NOT_GIVEN and maitai_config.max_tokens is not None:
             max_tokens = maitai_config.max_tokens
+        if max_completion_tokens and max_completion_tokens != openai.NOT_GIVEN:
+            max_tokens = max_completion_tokens
         if n == openai.NOT_GIVEN:
             n = maitai_config.n
         if frequency_penalty == openai.NOT_GIVEN:
@@ -313,6 +338,8 @@ class Completions:
                     if resp.warning:
                         warnings.warn(resp.warning, InferenceWarning)
                     if resp.error:
+                        if "Unknown error occurred" in resp.error:
+                            raise MaitaiConnectionError(resp.error)
                         raise InferenceException(resp.error)
                     timing.request_id = resp.chat_completion_response.request_id
                     timing.time_request_end = time.time()
@@ -497,6 +524,7 @@ def _process_inference_stream(
                 if chunk.choices and chunk.choices[0].finish_reason:
                     timing.time_request_end = time.time()
                     maitai.Inference.store_request_timing_data(timing)
+                if chunk.usage:
                     return
     except MaitaiConnectionError as e:
         if "gpt" not in retry_args.get("model", "") and "gpt" in retry_args.get(
